#! /usr/bin/perl -w

# Copyright (C) 2000 Colin Watson <cjw44@flatline.org.uk>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
# 
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
# 
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software Foundation,
#   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

# Shared functions for groupie programs to read and parse groupie.lists, and to
# help parse their own configuration files.

package Groupie::Lib;
require 5.004;	# my() in control structures, 'use constant'
use strict;

use Exporter;
use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
@ISA = qw(Exporter);
@EXPORT = qw(MAILTONEWS NEWSTOMAIL set_lists);
my @EXPORT_MAILTONEWS = qw(addresses_to_groups lists_addresses_to_groups);
my @EXPORT_NEWSTOMAIL = qw(groups_to_addresses lists_groups_to_addresses);
@EXPORT_OK = (@EXPORT_MAILTONEWS, @EXPORT_NEWSTOMAIL);
%EXPORT_TAGS = (
    all => [@EXPORT, @EXPORT_OK],
    mailtonews => [@EXPORT, @EXPORT_MAILTONEWS],
    newstomail => [@EXPORT, @EXPORT_NEWSTOMAIL],
);

use Groupie::Log;

my $lists = '@sysconfdir@/groupie.lists';

# Direction constants.
use constant MAILTONEWS => 0;
use constant NEWSTOMAIL => 1;

# If it turns out to be useful, a sub like this might pre-parse groupie.lists.
# At the moment, though, that file is typically only searched once per program
# invocation, so it wouldn't gain anything. Maybe if daemon mode gets
# implemented then it might prove useful.
sub set_lists ($)
{
    $lists = shift;
}

# Takes a shell-like pattern, with * for an anonymous wildcard and (name) for a
# named wildcard, and returns an equivalent compiled regular expression and a
# list of subexpression names.
sub pattern_to_regex ($)
{
    my $pattern = shift;
    my $regex = '';
    my @names = ();
    my %namemap = ();
    my $count = 1;
    local $_;
    foreach (split /(\*|\(.*?\))/, $pattern)
    {
	if ($_ eq '*')
	{
	    $regex .= '(.*?)';
	    push @names, $count++;
	}
	elsif ($_ =~ /\((.*?)\)/)
	{
	    if (exists $namemap{"_$1"})
	    {
		$regex .= '\\' . $namemap{"_$1"};
	    }
	    else
	    {
		$regex .= '(.*?)';
		$namemap{"_$1"} = $count;
	    }
	    push @names, "_$1";
	    $count++;
	}
	else
	{
	    $regex .= quotemeta;
	}
    }
    # Could use qr//, but it breaks 5.004, and is unlikely to provide enough of
    # a speedup here to be worth it. Use eval q{} and /o if the result is to be
    # used in a loop.
    return ($regex, @names);
}

# Parse any flags at the end of a line, and work out what should be filled into
# wildcards as a result.
sub parse_flags ($@)
{
    my $flagstr = shift;
    my @flags = split ' ', $flagstr;

    my @valid_flags = @_;
    my %subst;
    my $flag;

    # Parse the flags string.
    my %do_flag = ();
    foreach $flag (@valid_flags)
    {
	$do_flag{$flag} = 1;
    }
    while (defined($flag = shift @flags))
    {
	if (lc $flag eq 'subst')
	{
	    my ($name, $mail, $news) = splice @flags, 0, 3;
	    unless (exists $do_flag{subst})
	    {
		logwarn '"subst" flag not valid here (%s)', $flagstr;
		next;
	    }
	    $subst{$name} = [$mail, $news];
	}
	else
	{
	    logwarn 'Unrecognized flag %s (%s)', $flag, $flagstr;
	}
    }

    # Finally, return a hash of the structures we've created.
    return (subst => \%subst);
}

# Apply substitution information to a set of wildcards.
sub apply_subst ($$\@@)
{
    my ($direction, $subst, $names) = splice @_, 0, 3;

    for (my $i = 0; $i < @$names; $i++)
    {
	my $pairsref = $subst->{$names->[$i]};
	next unless defined $pairsref;
	my @pairs = @$pairsref;
	my ($mail, $news);
	while ($mail = shift @pairs and $news = shift @pairs)
	{
	    if ($direction == MAILTONEWS)   { $_[$i] =~ s/\Q$mail\E/$news/; }
	    else			    { $_[$i] =~ s/\Q$news\E/$mail/; }
	}
    }
}

# Alternate elements from the arguments and join them together. Used to merge
# wildcard replacements back into the original pattern.
sub merge_wildcards (\@\@)
{
    my @target = @{$_[0]};
    my @wildcards = @{$_[1]};
    my @merged = ();
    while (@target && @wildcards)
    {
	push @merged, (shift @target), (shift @wildcards);
    }
    push @merged, shift @target if @target;
    return join '', @merged;
}

# Replace a list of wildcard names with whatever they matched in the source
# pattern.
sub replace_names (\@\@\@)
{
    my ($source, $source_names, $target_names) = @_;
    my @target = ();
    my %source_index = ();
    local $_;
    for (0 .. $#$source_names)
    {
	$source_index{$source_names->[$_]} = $_;
    }
    for (@$target_names)
    {
	my $index = $source_index{$_};
	if (defined $index)
	{
	    push @target, $source->[$index];
	}
	else
	{
	    logwarn 'Target wildcard name %s not found in source pattern', $_;
	    push @target, '';
	}
    }
    return @target;
}

# Give a configuration file line (with mail pattern, news pattern, and optional
# flags) and all message headers to search, and this will return a list of the
# newsgroups to post to.
sub addresses_to_groups ($@)
{
    my $config = shift;
    my @groups = ();
    my ($mail, $news, $flags) = split ' ', $config, 3;
    $flags = '' unless defined $flags;
    my ($mailre, @mailnames) = pattern_to_regex $mail;
    my ($newsre, @newsnames) = pattern_to_regex $news;
    my @splitnews = split /\*|\(.*?\)/, $news, -1;

    foreach my $header (@_)
    {
	next unless defined $header;
	my $folded = $header;
	$folded =~ s/\n\s+/ /g;
	my @wildcards = ($folded =~ /$mailre/);
	# m// returns (1) if it matches but there are no parentheses, so
	# make sure @wildcards is the right length.
	my $matches = scalar @wildcards;
	if (@wildcards == 1 && $wildcards[0] eq '1' &&
	    (not defined $1 or $1 ne '1'))
	{
	    @wildcards = ();
	}
	next unless $matches;

	my %flags = parse_flags $flags, qw(subst);
	apply_subst MAILTONEWS, $flags{subst}, @mailnames, @wildcards;
	@wildcards = replace_names @wildcards, @mailnames, @newsnames;
	my $grouplist = merge_wildcards @splitnews, @wildcards;
	push @groups, split /,/, $grouplist;
    }

    return @groups;
}

# Give all message headers to search, and this will return a list of the
# newsgroups to post to, using the lists file. If the lists file is specified
# as "none" or otherwise could not be read, returns (undef).
sub lists_addresses_to_groups (@)
{
    my @headers = @_;
    my @groups = ();

    local *LISTS;
    return undef if $lists eq 'none';
    unless (open LISTS, "< $lists")
    {
	logwarn "Couldn't open %s: %s", $lists, $!;
	return undef;
    }

    local $_;
    while (<LISTS>)
    {
	chomp;
	s/#.*//;
	if (/\\$/)
	{
	    $_ .= <LISTS>;
	    chomp;
	    redo;
	}
	next unless m/\S/;
	push @groups, addresses_to_groups $_, @headers;
    }

    return @groups;
}

# TODO: [lists_]groups_to_addresses().

1;
