#! /usr/bin/perl -w

# Copyright (C) 2000 Colin Watson <cjw44@flatline.org.uk>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
# 
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
# 
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software Foundation,
#   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

# Logging functions for groupie programs.
# 
# Be careful about editing this file, as logerror() is called by the die()
# handler - that is, any errors while reading this module will cause errors to
# be sent to standard error, which is probably not what you want.

package Groupie::Log;
use strict;

use Exporter;
use vars qw(@ISA @EXPORT %EXPORT_TAGS);
@ISA = qw(Exporter);
@EXPORT = qw(setlogfile setlogname loginfo logwarn logerror die_handler);
%EXPORT_TAGS = (all => [@EXPORT]);

use Sys::Syslog;

my $logfile = 'syslog';
my $logname = 'groupie';

sub setlogfile ($);
sub setlogname ($);
sub _printlog ($$$@);
sub loginfo ($@);
sub logwarn ($@);
sub logerror ($$@);
sub die_handler;

sub setlogfile ($)
{
    $logfile = shift;
}

sub setlogname ($)
{
    $logname = shift;
}

my %prefix = (info => 'Info', warning => 'Warning', err => 'Error');

sub _printlog ($$$@)
{
    my ($method, $priority, $format, @args) = @_;
    unless ($priority =~ /^(?:info|warning|err)$/)
    {
	# Be careful, recursion.
	_printlog 'syslog', 'warning', 'Invalid priority level: %s', $priority;
    }
    if (not defined $method or $method =~ /^(?:none|syslog)?$/)
    {
	# 5.004's Sys::Syslog didn't export setlogsock().
	Sys::Syslog::setlogsock 'unix';
	openlog $logname, 'pid', 'news';
	syslog $priority, $format, @args;
	closelog;
    }
    elsif ($method eq 'stderr')
    {
	printf STDERR "%s: $format\n", $prefix{$priority}, @args;
    }
    else
    {
	local *LOGFILE;
	if (open LOGFILE, ">> $method")
	{
	    printf LOGFILE "%s: $format\n", $prefix{$priority}, @args;
	    close LOGFILE;
	}
	else
	{
	    # Be careful, recursion.
	    _printlog 'syslog', 'warning',
		      "Couldn't open logfile: %s", $method;
	}
    }
}

sub loginfo ($@)
{
    my $verbose = $Groupie::ConfigFile::config{'verbose'};
    return if defined $verbose and $verbose < 2;
    my ($format, @args) = @_;
    _printlog $logfile, 'info', $format, @args;
}

sub logwarn ($@)
{
    my $verbose = $Groupie::ConfigFile::config{'verbose'};
    return if defined $verbose and $verbose < 1;
    my ($format, @args) = @_;
    _printlog $logfile, 'warning', $format, @args;
}

# This is called from the $SIG{__DIE__} handler, so be very, very careful not
# to break this.
sub logerror ($$@)
{
    my ($exitcode, $format, @args) = @_;
    _printlog $logfile, 'err', $format, @args;
    if (exists $Groupie::ConfigFile::config{'suppress-exitcode'} and
	$Groupie::ConfigFile::config{'suppress-exitcode'} == 0)
    {
	exit $exitcode;
    }
    else
    {
	exit 0;
    }
}

1;
